import numpy
from numpy.fft import fftshift, ifftshift
import matplotlib.pyplot as plt

from lib6003.image import *

def pad(array, r_pad=1, c_pad=1, mode='extend'):
    """
    Given a numpy array, make a larger array by "padding" the edges of the
    array (adding r_pad number of pixels to the top and bottom of the array,
    and c_pad number of pixels to the left and right of the array).

    If mode is 'extend', then the values on the edge of the array are repeated
    in the padded region.  If mode is 'zero', then the array will instead be
    padded by zeros.
    """
    if mode == 'extend':
        _nmode = 'edge'
    elif mode == 'zero':
        _nmode = 'constant'
    else:
        raise ValueError('Invalid mode for pad: %s' % mode)
    return ifftshift(numpy.pad(fftshift(array), ((r_pad, r_pad), (c_pad, c_pad)), _nmode))


def unpad(array, r_pad=1, c_pad=1):
    """
    An inverse operation to pad.  Removes edges that were added by padding.
    """
    assert r_pad != 0 and c_pad != 0, "Padding values must be nonzero"
    return ifftshift(fftshift(array)[r_pad:-r_pad, c_pad:-c_pad])


def convolve_2d_fft(image, kernel, pad_edges=False):
    """
    Convolve `image` with `kernel` by operating in the frequency domain.  Both
    are represented as NumPy arrays.  This code assumes that `kernel` is at most
    as big as `image`.

    If pad_edges is True, first pad the edges of the image (in 'extend' mode)
    and unpad at the end (to prevent any edge effects from being visible).
    Otherwise, do not pad the image at all (in this case, boundary effects may
    be seen).

    Return a NumPy array containing the result of the convolution.

    Note that this function assumes that the point [0,0] represents the center
    (not the topleft) of the kernel.
    """
    kh, kw = kernel.shape
    if pad_edges:
        image = pad(image, kh//2, kw//2)
    ih, iw = image.shape
    kernel = pad(kernel, (ih-kh)//2, (iw-kw)//2, 'zero')
    result = image.size*ifft2(fft2(image)*fft2(kernel))
    if pad_edges:
        result = unpad(result, kh//2, kw//2)
    return result


